"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var react_1 = require("react");
// handle canvas element resize, and pixel density as per https://developer.mozilla.org/en-US/docs/Web/API/Window/devicePixelRatio#correcting_resolution_in_a_canvas
var resizeCanvas = function (canvas, contextType, forceResize) {
    if (forceResize === void 0) { forceResize = false; }
    var resized = false;
    // TODO use a seemingly more accurate method, as described here: https://webglfundamentals.org/webgl/lessons/webgl-resizing-the-canvas.html
    var _a = canvas.getBoundingClientRect(), width = _a.width, height = _a.height;
    if (forceResize || canvas.width !== width || canvas.height !== height) {
        var scale = (window === null || window === void 0 ? void 0 : window.devicePixelRatio) ? window.devicePixelRatio : 1; // if not present, assume pixel ratio 1
        var context = canvas.getContext(contextType);
        var newWidth = Math.floor(width * scale);
        var newHeight = Math.floor(height * scale);
        if (context) {
            resized = true;
            canvas.width = newWidth;
            canvas.height = newHeight;
            // this call is needed to normalise the coordinate system to use CSS pixels
            if ('scale' in context) {
                context.scale(scale, scale);
            }
        }
    }
    return resized;
};
var useCanvas = function (_a) {
    var contextType = _a.contextType, draw = _a.draw;
    var _b = (0, react_1.useState)(true), firstDraw = _b[0], setFirstDraw = _b[1];
    // keep a typed ref to the Canvas DOM element using generics
    var canvasRef = (0, react_1.useRef)(null);
    // for simplicity, we also keep a typed ref to the context, which can either be 2D or 3D (WebGL 1.0 or 2.0)
    var canvasContextRef = (0, react_1.useRef)(null);
    (0, react_1.useEffect)(function () {
        var _a;
        // to provide the frame update to the draw function
        var frameCount = 0;
        // to be able to calculate amount of time passed between draws
        var lastDrawTimestamp;
        // to cancel the animation frame request via clean up function
        var animFrameId;
        // initialise the context ref, which can only be done once the DOM is ready
        if (canvasRef.current) {
            // the first time we want to force resizing to make sure pixel ratio adjustment is performed
            resizeCanvas(canvasRef.current, contextType, firstDraw);
            setFirstDraw(false);
            canvasContextRef.current = (_a = canvasRef.current) === null || _a === void 0 ? void 0 : _a.getContext(contextType);
            // to progressively update frames
            var updateFrame_1 = function () {
                frameCount++;
                if (canvasContextRef.current) {
                    var currentTimestamp = new Date().getTime();
                    var elapsedTime = lastDrawTimestamp > 0 ? currentTimestamp - lastDrawTimestamp : 0;
                    draw(canvasContextRef.current, frameCount, elapsedTime);
                    lastDrawTimestamp = currentTimestamp;
                }
                animFrameId = window.requestAnimationFrame(updateFrame_1);
            };
            // start the frame loop as soon as the browser is ready to draw
            requestAnimationFrame(updateFrame_1);
        }
        // clean up when the component is unmounted
        return function () {
            window.cancelAnimationFrame(animFrameId);
        };
    }, [firstDraw, draw, contextType]);
    return canvasRef;
};
exports.default = useCanvas;
